@@ -14,6 +14,10 @@ from .logger import setup_logger
 from .check import CheckBase
 
 
+class SendDataException(Exception):
+    pass
+
+
 def _convert_verify_ssl(val):
     if val is None or val.lower() in ['true', '1', 'y', 'yes']:
         return None  # None for default SSL check
@@ -63,22 +67,24 @@ class Agent:
 
         self._loop: Optional[asyncio.AbstractEventLoop] = None
         self._get_headers = {'Authorization': f'Bearer {token}'}
-        self._post_headers = {'Content-Type': 'application/json'}
-        self._post_headers.update(self._get_headers)
+        self._json_headers = {'Content-Type': 'application/json'}
+        self._json_headers.update(self._get_headers)
 
         self.asset_id: Optional[int] = None
         self.api_uri: str = os.getenv('API_URI', 'https://api.infrasonar.com')
         self.verify_ssl = _convert_verify_ssl(os.getenv('VERIFY_SSL', '1'))
         self._read_json()
 
-    async def announce(self, asset_name: Optional[str] = None):
+    async def announce(self, asset_name: Optional[str] = None,
+                       asset_kind: Optional[str] = None):
         """Announce the agent.
 
         Argument `asset_name` is only used if the agent is new (no asset Id
         exists) and if not given, the fqdn is used."""
         try:
             if self.asset_id is None:
-                self.asset_id, name = await self._create_asset(asset_name)
+                self.asset_id, name =\
+                     await self._create_asset(asset_name, asset_kind)
                 self._dump_json()
                 logging.info(f'created agent {name} (Id: {self.asset_id})')
                 return
@@ -128,7 +134,7 @@ class Agent:
             data["runtime"] = runtime
 
         try:
-            async with ClientSession(headers=self._post_headers) as session:
+            async with ClientSession(headers=self._json_headers) as session:
                 async with session.post(
                     url,
                     json=data,
@@ -140,16 +146,20 @@ class Agent:
 
         except Exception as e:
             msg = str(e) or type(e).__name__
-            logging.error(
-               'failed to send data for '
-               f'check {check_key}: {msg} (url: {url})')
+            raise SendDataException(
+                f'failed to send data ({check_key}): {msg} (url: {url})')
 
     def start(self, checks: Iterable[CheckBase],
-              asset_name: Optional[str] = None):
+              asset_name: Optional[str] = None,
+              asset_kind: Optional[str] = None):
         """Start the agent demonized.
 
         The `asset_name` argument is only used on the accounce when the asset
-        is new and must be created. If not given, the fwdn is used.
+        is new and must be created. If not given, the fqdn is used.
+
+        The `asset_kind` argument is only used on the accounce when the asset
+        is new and must be created. If not given, the asset will be created
+        with the default `Asset` kind.
 
         Argument `checks` must be an iterable containing subclasses of
         CheckBase. (the classes, not instances of the class)
@@ -159,7 +169,8 @@ class Agent:
 
         self._loop = asyncio.get_event_loop()
         try:
-            self._loop.run_until_complete(self._start(checks, asset_name))
+            self._loop.run_until_complete(
+                self._start(checks, asset_name, asset_kind))
         except Exception:
             self._loop.run_until_complete(self._loop.shutdown_asyncgens())
             self._loop.close()
@@ -171,8 +182,9 @@ class Agent:
             task.cancel()
 
     async def _start(self, checks: Iterable[CheckBase],
-                     asset_name: Optional[str] = None):
-        await self.announce(asset_name)
+                     asset_name: Optional[str] = None,
+                     asset_kind: Optional[str] = None):
+        await self.announce(asset_name, asset_kind)
         checks = [self._check_loop(c) for c in checks]
         try:
             await asyncio.wait(checks)
@@ -206,6 +218,8 @@ class Agent:
                 await self.send_data(check.key, check_data)
             except asyncio.TimeoutError:
                 logging.error(f'check error ({check.key}): timed out')
+            except SendDataException as e:
+                logging.error(str(e))
             except Exception as e:
                 msg = str(e) or type(e).__name__
                 logging.error(f'check error ({check.key}): {msg}')
@@ -215,7 +229,8 @@ class Agent:
                 ts = time.time()
                 ts_next += check.interval
 
-    async def _create_asset(self, asset_name: Optional[str] = None) -> int:
+    async def _create_asset(self, asset_name: Optional[str] = None,
+                            asset_kind: Optional[str] = None) -> int:
         url = _join(self.api_uri, 'container/id')
         async with ClientSession(headers=self._get_headers) as session:
             async with session.get(url, ssl=self.verify_ssl) as r:
@@ -229,7 +244,7 @@ class Agent:
         url = _join(self.api_uri, f'container/{container_id}/asset')
         name = _fqdn() if asset_name is None else asset_name
         data = {"name": name}
-        async with ClientSession(headers=self._post_headers) as session:
+        async with ClientSession(headers=self._json_headers) as session:
             async with session.post(url, json=data, ssl=self.verify_ssl) as r:
                 if r.status != 201:
                     msg = await r.text()
@@ -240,7 +255,7 @@ class Agent:
 
         try:
             url = _join(self.api_uri, f'asset/{asset_id}/collector/{self.key}')
-            async with ClientSession(headers=self._post_headers) as session:
+            async with ClientSession(headers=self._json_headers) as session:
                 async with session.post(url, ssl=self.verify_ssl) as r:
                     if r.status != 204:
                         msg = await r.text()
@@ -249,6 +264,22 @@ class Agent:
             msg = str(e) or type(e).__name__
             logging.error(f'failed to assign collector: {msg}')
 
+        if asset_kind:
+            data = {"kind": asset_kind}
+            try:
+                url = _join(self.api_uri, f'asset/{asset_id}/kind')
+                async with ClientSession(
+                        headers=self._json_headers) as session:
+                    async with session.patch(url,
+                                             json=data,
+                                             ssl=self.verify_ssl) as r:
+                        if r.status != 204:
+                            msg = await r.text()
+                            raise Exception(f'{msg} (error code: {r.status})')
+            except Exception as e:
+                msg = str(e) or type(e).__name__
+                logging.error(f'failed to set asset kind: {msg}')
+
         return asset_id, name
 
     def _read_json(self):
